#include <stdio.h>
#include <string.h>
#include "type.h"

Atom *
dictionary ()
{
    return NULL;
}

Atom *
define (Atom *dict, char *name, Atom *value)
{
    Atom *result;

    /*
     * if dict is nil, create an empty list and add this to the list
     */
    if (nullp (dict))
	return cons (cons (string (name), value), NULL);
    /*
     * if dict is not a list, it's an error
     */
    else if (!listp (dict))
	return NULL;
    /*
     * if the first element of dict matches, replace it with the new definition
     */
    else if (stringp (this (this (dict))) && strcmp (stringval (this (this (dict))), name) == 0)
	return cons (cons (string (name), value), next (dict));
    /*
     * otherwise add this entry to the rest of the list
     */
    else {
	set_next (dict, define (next (dict), name, value));
	return dict;
    }
}

/*
 * search 'dict' for a (name value) pair matching 'name',
 * and return the (name value) pair.
 *
 * XXX should just return the value
 */

Atom *
lookup (Atom *dict, char *name)
{
    if (nullp (dict))
	return NULL;
    else if (!listp (dict))
	return NULL;
    else if (stringp (this (this (dict))) && strcmp (stringval (this (this (dict))), name) == 0)
	return this (dict);
    else
	return lookup (next (dict), name);
}

/*
 * like 'lookup' except that it returns only the value
 */

Atom *
nlookup (Atom *dict, char *name)
{
    Atom *result = lookup (dict, name);

    if (result)
	return next (result);
    else
	return NULL;
}

void
fprint_dict (FILE *fp, Atom *dict)
{
    if (nullp (dict))
	return;
    fprintf (fp, "%s:", stringval (this (this (dict))));
    fprint_atom (fp, next (this (dict)));
    fprintf (fp, "\n");
    fprint_dict (fp, next (dict));
}

#if 0

/*
 * XXX this will eat up one stack activation record for every line in
 * the file in order to keep the list structure opaque and avoid
 * the overhead of append().  is this evil?  probably.
 */

static Atom *
read_dict_buf (FILE *fp, char *buf, int size)
{
    char *colon = strchr (buf, ':');

    if (fgets (buf, size, fp) == NULL)
	return NULL;

    if (colon) {
	Atom *name = stringn (buf, colon-buf);
	Atom *value = string (colon+1);

	return cons (cons (name, value), read_dict_buf (fp, buf, size));
    }
}

Atom *
read_dict (FILE *fp)
{
    Atom *result;
    char buf[10240];

    result = read_dict_buf (fp, buf, sizeof (buf));
}
#endif

/*
 * read a file containing name,value pairs and produce a dictionary
 *
 * syntax:
 * name#value	define value as an integer
 * name:value	define value as a string
 * 
 * XXX untested
 * XXX needs to be able to read arbitrary data types, not just
 * strings and integers
 */

Atom *
read_dict (FILE *fp)
{
    char buf[10240];
    Atom *result = NULL;

    while (fgets (buf, sizeof (buf), fp)) {
	char *sep;
	char *nl;

	if (nl = strchr (buf, '\n'))
	    *nl = '\0';

	for (sep = buf; *sep; ++sep) {
	    if (*sep == ':' || *sep == '#')
		break;
	}
	
	if (*sep) {
	    Atom *value;

	    if (*sep == ':') {
		*sep = '\0';
		value = string (sep+1);
		result = define (result, buf, value);
	    }		
	    else {
		*sep = '\0';
		value = integer (atoi (sep+1));
		result = define (result, buf, value);
	    }
	}
    }
    return result;
}

/*
 * write a dict out to a file that can be read by read_dict()
 * XXX needs to be able to write arbitrary data, not just strings and ints
 */

void
write_dict (FILE *fp, Atom *dict)
{
    for (; !nullp (dict); dict = next (dict)) {
	if (stringp (this (this (dict)))) {
	    if (stringp (next (this (dict)))) {
		fprintf (fp, "%s:%s\n",
			 stringval (this (this (dict))),
			 stringval (next (this (dict))));
	    }
	    else if (integerp (next (this (dict)))) {
		fprintf (fp, "%s#%d\n",
			 stringval (this (this (dict))),
			 intval (next (this (dict))));
	    }
	}
    }
}



#ifdef TEST
main ()
{
    Atom *a;

    a = dictionary ();
    a = define (a, "one", string ("one"));
    fprint_atom (stdout, a);
    printf ("\n");
    a = define (a, "two", string ("two"));
    fprint_atom (stdout, a);
    printf ("\n");
    a = define (a, "three", string ("three"));
    fprint_atom (stdout, a);
    printf ("\n");
    a = define (a, "one", integer (1));
    fprint_atom (stdout, a);
    printf ("\n");
    a = define (a, "three", integer (3));
    fprint_atom (stdout, a);
    printf ("\n");
    a = define (a, "two", integer (2));
    fprint_atom (stdout, a);
    printf ("\n");
    a = define (a, "four", integer (4));
    fprint_atom (stdout, a);
    printf ("\n");
}
#endif
