#include <stdio.h>
#include <stdarg.h>
#include <string.h>

/*
 * given a NULL-terminated actual parameter list,
 * return a vector to malloc'ed copies of those arguments
 */

char **
make_argv (char *arg0, ...)
{
    int argc;
    char **result;
    char *s;
    va_list ap;

    if (arg0 == NULL) {
        result = (char **) xalloc (sizeof (char *));
        result[0] = NULL;
        return result;
    }

    /* count arguments */       
    argc = 1;                   /* for arg0 */
    va_start (ap, arg0);
    while (s = va_arg (ap, char *))
        ++argc;
    va_end (ap);

    result = (char **) xalloc ((argc + 2) * sizeof (char *));

    argc = 0;                   /* for arg0 */
    result[argc++] = strdup (arg0);
    va_start (ap, arg0);
    while (result[argc] = va_arg (ap, char *)) {
        result[argc] = strdup (result[argc]);
        ++argc;
    }
    va_end (ap);
    result[argc] = NULL;

    return result;
}

/*
 * free up a malloc'ed vector (terminated with NULL) of malloc'ed strings.
 */

void
free_argv (char **argv)
{
    int i;

    if (argv == NULL)
        return;
    for (i = 0; argv[i]; ++i) {
        free (argv[i]);
        argv[i] = NULL;
    }
    free (argv);
}
