#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include "conf.h"
#include "dirname.h"

/*
 * create a directory (if it is missing)
 * along with any necessary ancestors
 */

int
mkdir_recursive (char *dir, int mode)
{
    struct stat st;
    extern int verbosity;

#if 0
   if (verbosity > 0)
	fprintf (stderr, "mkdir_recursive (\"%s\", %o)\n", dir, mode);
#endif

    if (*dir == '\0' || strcmp (dir, "/") == 0)
	return 0;
    if (stat (dir, &st) < 0) {
	int status;
	char *parent;

	/*
	 * stat failed.  assume that the directory does not exist, 
 	 * so create any ancestors that are needed and then create 
	 * this directory.  
         * (stat could also fail for other reasons, for instance,
	 * if some ancestor of this directory were not searchable.
	 * if that happens we will take longer to fail, but we will
	 * still fail in a reasonable amount of time.)
	 */
	parent = strdup (my_dirname (dir));
	status = mkdir_recursive (parent, mode);
	if (status == 0)
	    if (status = mkdir (dir, mode) < 0) {
		fprintf (stderr, "mkdir (%s, 0%o) => %s\n", dir, mode,
			 strerror (errno));
	    }
	free (parent);
	return status;
    }
    else if ((st.st_mode & S_IFMT) != S_IFDIR) {
	/*
	 * file exists, but is not a directory
	 */
	fprintf (stderr, "mkdir_recursive (%s) => not a directory\n", dir);
	errno = ENOTDIR;
	return -1;
    }
    else {
	/* directory already exists; do nothing */
#if 0
	fprintf (stderr, "mkdir_recursive (%s) => 0\n", dir);
#endif
	return 0;
    }
}

#if TEST
int verbosity = 1;

main (int argc, char **argv)
{
    mkdir_recursive (argv[1], 0711);
}
#endif
