/*
 * routines for dealing with netbuild.index files
 */

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <libgen.h>
#include <string.h>
#include "cf.h"
#include "concat.h"
#include "dict.h"
#include "expr.h"
#include "field.h"
#include "talloc.h"

/*
 * netbuild.index file syntax:
 *
 *
 * [stanza1]
 * name1:def1
 * name2:def2
 * [stanza2]
 * name1:def1
 * name3:def3
 * 
 * should produce
 * 
 * ((stanza1 (name1 def1) (name2 def2)))
 * ((stanza2 (name1 def1) (name3 def2))))
 */

     
/* return 1 iff 's' contains nothing but white space */

static int
blankline (char *s)
{
    while (isspace (*s))
	++s;
    return (*s == '\0');
}

/* change all NL and CR characters to SP, remove trailing SPs */

static void
cleanup (char *s)
{
    char *p = s;

    while (*p) {
	if (*p == '\n' || *p == '\r')
	    *p = ' ';
	++p;
    }

    /* now remove trailing spaces */
    while (p > s && p[-1] == ' ')
	*--p = '\0';
}

/* return a pointer to the first non-space char in 's' */

static char *
firstnonspace (char *s)
{
    while (isspace (*s))
	++s;
    return s;
}

Atom *
parse_nb_index_file (char *uri, FILE *fp)
{
    char buf[10240];
    char *ptr;
    Atom *result = NULL;
    Atom *stanza = NULL;
    char *filename = NULL;
    int lineno = 1;
    int copying = 0;
    char *base_uri;

    base_uri = strdup (uri); 
    if (ptr = strrchr (base_uri, '/'))
	*ptr = '\0';

    while (fgets (buf, sizeof (buf), fp)) {
	cleanup (buf);
	if (blankline (buf))
	    ;
	else if (*buf == '[') {			/* [compiler] */
	    /* close up old stanza and append to result */

	    if (filename && stanza) {
		if (nlookup (stanza, "library.precedence") == NULL)
		    stanza = define (stanza, "library.precedence",
				     string ("-9999"));
		result = define (result, filename, meta ("candidate", stanza));
	    }
	    if (filename)
		free (filename);
	    filename = NULL;
	    stanza = NULL;

	    /* remember new filename if this is a valid stanza */
	    if (buf[1] == '#')
		fprintf (stderr, "skipping %s\n", buf);
	    else if ((ptr = strchr (buf+1, ']')) == NULL)
		fprintf (stderr, "%s (%d): missing ']'\n",
			 filename, lineno);
	    else {
		*ptr = '\0';
		/* filename = strdup (buf+1); */
		filename = concat (base_uri, "/", buf+1, NULL);
	    }
	}
	else if (ptr = strchr (buf, ':')) {	/* name: value */
	    *ptr = '\0';
	    stanza = define (stanza, buf,
			     string (firstnonspace (ptr+1)));
	}
	else {
	    fprintf (stderr, "%s (%d): unrecognized netbuild.index line\n",
		     uri, lineno);
	    fprintf (stderr, "<%s>\n", buf);
	}
	lineno ++;
    }
    /* close up old stanza and append to result */
    
    if (filename && stanza) {
	if (nlookup (stanza, "library.precedence") == NULL)
	    stanza = define (stanza, "library.precedence",
			     string ("-9999"));
	result = define (result, filename, meta ("candidate", stanza));
    }
    if (filename)
	free (filename);
    filename = NULL;
    stanza = NULL;
    
    return result;
}
