#include <stdio.h>
#include "conf.h"
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#else
void *malloc ();
#endif
#include "talloc.h"

/*
 * allocate memory like malloc, except
 * - xalloc(0) returns a valid pointer
 * - failures cause the program to exit, on the theory that we're
 *   not likely to recover anyway.
 * any necessary cleanup can be arranged with onexit().
 */

void *
xalloc (unsigned int size)
{
    void *result;

    if (size == 0)
	size = 1;

    if ((result = malloc (size)) == NULL) {
	fprintf (stderr, "Out of memory!\n");
	exit (1);
    }
    return result;
}

/*
 * like strdup except that any failures cause immediate exit
 */

char *
xstrdup (char *s)
{
    char *result;
    int len;

    if (s == NULL)
	return NULL;
    result = xalloc (len = (strlen (s) + 1));
    memcpy (result, s, len);
    return result;
}

char *
xstrndup (char *s, int len)
{
    char *result;
    int slen;

    if (s == NULL)
	return NULL;
    
    slen = strlen (s);

    if (len < slen) {
	result = xalloc (len + 1);
	memcpy (result, s, len);
	result[len] = '\0';
    }
    else {
	result = xalloc (slen + 1);
	memcpy (result, s, slen);
    }
    return result;
}

#ifdef TEST
main (int argc, char **argv)
{
    char *foo;

    if (argc != 3) {
	printf ("usage: xstrndup string len\n");
	exit (1);
    }
    foo = xstrndup (argv[1], atoi(argv[2]));
    printf ("'%s' (%d)\n", foo, strlen (foo));
}
#endif
